<?php

/*
 * Plugin Info Widget.
 * This plugin displays all messages like new driven records and system messages into a
 * transparent widget instead within the chat. Just see the Screenshot for details.
 * ----------------------------------------------------------------------------------
 * Author:	undef.de
 * Version:	0.9.8
 * Date:	2010-10-24
 * Copyright:	2009 - 2010 by undef.de
 * Home:	http://www.undef.de/Trackmania/Plugins/
 * System:	xaseco/1.06 and up
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: none
 * Do not use this plugin in combination with "plugin.msglog.php"!
 */

/* The following manialink id's are used in this plugin (the 910 part of id can be changed on trouble):
 * 91000	id for manialinks
 * 91001	id for manialink MessageWindow
 */

Aseco::registerEvent('onStartup',			'iwid_onStartup');
Aseco::registerEvent('onNewChallenge',			'iwid_onNewChallenge');
Aseco::registerEvent('onPlayerConnect',			'iwid_onPlayerConnect');
Aseco::registerEvent('onPlayerDisconnect',		'iwid_onPlayerDisconnect');
Aseco::registerEvent('onPlayerManialinkPageAnswer',	'iwid_onPlayerManialinkPageAnswer');
Aseco::registerEvent('onEndRace1',			'iwid_onEndRace1');

Aseco::addChatCommand('toggleinfo',			'Toggle the display of the Info-Widget');

global $iwid_config, $IWID_MSGS;
$IWID_MSGS = array();

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onStartup
function iwid_onStartup ($aseco, $command) {
	global $iwid_config, $IWID_VERSION;

	if ( version_compare(XASECO_VERSION, 1.06, '<') ) {
		trigger_error('[plugin.info_widget.php] Not supported xaseco version ('. XASECO_VERSION .')! Please update to min. version 1.06!', E_USER_ERROR);
	}

	if (!$iwid_config = $aseco->xml_parser->parseXML('info_widget.xml')) {
		trigger_error("[plugin.info_widget.php] Could not read/parse config file info_widget.xml !", E_USER_ERROR);
	}
	$iwid_config = $iwid_config['INFO_WIDGET'];


	$iwid_config['MANIALINK_ID'] = '910';
	$iwid_config['VERSION'] = '0.9.8';


	$iwid_config['TEXT_SIZE'][0] = intval($iwid_config['TEXT_SIZE'][0]);
	$iwid_config['TEXT_SCALE'][0] = intval($iwid_config['TEXT_SCALE'][0]);
	$iwid_config['WIDGET'][0]['AUTO_HIDE'][0] = intval($iwid_config['WIDGET'][0]['AUTO_HIDE'][0]);
	$iwid_config['AMOUNT_MESSAGES'][0] = intval($iwid_config['AMOUNT_MESSAGES'][0]);

	if ($iwid_config['TEXT_SCALE'][0] == 0) {
		$iwid_config['TEXT_SCALE'][0] = 1;
	}
	if ($iwid_config['TEXT_SCALE'][0] > 1) {
		$iwid_config['TEXT_SCALE'][0] = 1;
	}

	if ($iwid_config['TEXT_SIZE'][0] <= 0) {
		$iwid_config['TEXT_SIZE'][0] = 1;
	}
	if ($iwid_config['TEXT_SIZE'][0] > 3) {
		$iwid_config['TEXT_SIZE'][0] = 1;
	}

	if ($iwid_config['WIDGET'][0]['AUTO_HIDE'][0] > 0) {
		$iwid_config['WIDGET'][0]['AUTO_HIDE'][0] = $iwid_config['WIDGET'][0]['AUTO_HIDE'][0] * 1000;
	}

	// Set multiplier for line-height of Textsizes
	$iwid_config['MULTIPLIER'] = array(2,2,5,9);

	$iwid_config['OPACITY'] = array('1','3','5','8','A','B','D','E','F');
	$iwid_config['STATUS'] = true;

	if (strtoupper($iwid_config['HIDE_NOTICE_MESSAGES'][0]) == 'TRUE') {
		setCustomUIField('notice', false);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerConnect
function iwid_onPlayerConnect ($aseco, $player) {
	global $iwid_config, $IWID_MSGS;

	// Init the Player Array
	$IWID_MSGS[$player->login] = array();

	$player->data['InfoWidgetStatus'] = 'visible';

	$message = '$0B3This is the plugin {#highlite}$L[http://www.undef.de/Trackmania/Plugins/]Info-Widget/'. $iwid_config['VERSION'] .'$L$0B3 for xaseco.'. LF .'$0B3To hide this message type {#highlite}/toggleinfo';
	send_window_message($aseco, $message, $player);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerDisconnect
function iwid_onPlayerDisconnect ($aseco, $player) {
	global $IWID_MSGS;

	// Remove Player log from array to save mem
	$tmp = array();
	foreach ($IWID_MSGS as $ply => $array) {
		if ($ply != $player->login) {
			$tmp[$ply] = $array;
		}
	}
	$IWID_MSGS = $tmp;
	unset($tmp);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onNewChallenge
function iwid_onNewChallenge ($aseco) {
	global $iwid_config;

	$iwid_config['STATUS'] = true;

	// display the messages
	foreach ($aseco->server->players->player_list as $player) {
		iwid_displayMessageWindow($aseco, $player);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onEndRace1
function iwid_onEndRace1 ($aseco) {
	global $iwid_config;

	$iwid_config['STATUS'] = false;

	// Hide the Widget at all Players
	iwid_hideMessageWindow($aseco, false);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// Save the messages into an array
// $param can be 'true', 'false' or the player array
function send_window_message ($aseco, $message, $param = false) {
	global $iwid_config, $IWID_MSGS;

	// If $param is be the player array, send the message to the given player
	if ( ($param !== true) && ($param !== false) && ($param->id) ) {

		// Add message to Player
		iwid_addMessageToPlayer($aseco, $param, $message);

		if ($iwid_config['STATUS'] == true) {
			iwid_displayMessageWindow($aseco, $param);
		}
	}
	else {
		// Display the messages to all players
		foreach ($aseco->server->players->player_list as $player) {

			// Add message to Player
			iwid_addMessageToPlayer($aseco, $player, $message);

			if ($iwid_config['STATUS'] == true) {
				iwid_displayMessageWindow($aseco, $player);
			}
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// Add an new message to the Player Array
function iwid_addMessageToPlayer ($aseco, $player, $message) {
	global $iwid_config, $IWID_MSGS;

	// Transform CRLF to LF
	$message = str_replace("\r", '', $message);

	// Add the new message
	$msg = split(LF, $message);
	if ( is_array($msg) ) {
		foreach ($msg as &$text) {
			$text = iwid_cleanUp($text);
		}
		$IWID_MSGS[$player->login] = array_merge($IWID_MSGS[$player->login], $msg);
	}
	else {
		array_push($IWID_MSGS[$player->login], iwid_cleanUp($msg));
	}
	unset($msg);


	// If more then $iwid_config['AMOUNT_MESSAGES'][0] messages in the array, get the last $iwid_config['AMOUNT_MESSAGES'][0]
	if ( count($IWID_MSGS[$player->login]) > $iwid_config['AMOUNT_MESSAGES'][0]) {
		$IWID_MSGS[$player->login] = array_slice($IWID_MSGS[$player->login], (count($IWID_MSGS[$player->login]) - $iwid_config['AMOUNT_MESSAGES'][0]), $iwid_config['AMOUNT_MESSAGES'][0]);
	}


	// If less then $iwid_config['AMOUNT_MESSAGES'][0], just fill up
	if ( count($IWID_MSGS[$player->login]) < $iwid_config['AMOUNT_MESSAGES'][0]) {
		$tmp = array();
		for ($i = count($IWID_MSGS[$player->login]); $i < $iwid_config['AMOUNT_MESSAGES'][0]; $i++) {
			array_push($tmp, ' ');
		}
		$IWID_MSGS[$player->login] = array_merge($tmp, $IWID_MSGS[$player->login]);
		unset($tmp);
	}

	// If message window is hidden for this player, so just show all messages in chat too.
	if ($player->data['InfoWidgetStatus'] == 'hidden') {
		$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $player->login);
	}

}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function iwid_cleanUp ($message) {
	// clean up for xml
	// http://en.tm-wiki.org/wiki/Text_Formatting
	$message = preg_replace('/^(\{#[A-Z]+\}|\$[A-F0-9TISWNMGZOHLP\$]+)(>|>>) (.*)$/i', '$1$3', $message);	// remove > or >> from message or all colors and formating codes
	$message = str_ireplace('$S', '', $message);								// remove $s case-insensitive
	$message = str_replace('"', '&quot;', $message);
	$message = str_replace("'", '&apos;', $message);
	$message = str_replace('<', '&lt;', $message);
	$message = str_replace('>', '&gt;', $message);

	return $message;
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// displays the message window
function iwid_displayMessageWindow ($aseco, $player) {
	global $iwid_config, $IWID_MSGS;

	// When the player wish to hide this window, bail out.
	if ($player->data['InfoWidgetStatus'] == 'hidden') {
		return;
	}

	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks id="'. $iwid_config['MANIALINK_ID'] .'00">';
	$xml .= '<manialink id="'. $iwid_config['MANIALINK_ID'] .'01">';
	$xml .= '<frame posn="'. $iwid_config['WIDGET'][0]['WIDGET_POS_X'][0] .' '. $iwid_config['WIDGET'][0]['WIDGET_POS_Y'][0] .' 0">';
	$line = 0;
	$opacity = 0;
	foreach ($IWID_MSGS[$player->login] as $msg) {
		if (strtoupper($iwid_config['TEXT_SHADOW'][0]) == 'TRUE') {
			$msg = '$S'. $msg;
		}
		$xml .= '<label sizen="79.6 0" posn="0 '. -$line .' 0" scale="'. $iwid_config['TEXT_SCALE'][0] .'" textsize="'. $iwid_config['TEXT_SIZE'][0] .'" textcolor="FFF'. $iwid_config['OPACITY'][$opacity] .'" text="'. $aseco->formatColors($msg) .'"/>';
		$line += (($iwid_config['TEXT_SCALE'][0] / $iwid_config['TEXT_SIZE'][0]) * $iwid_config['MULTIPLIER'][$iwid_config['TEXT_SIZE'][0]]);
		$opacity += 1;
	}
	$xml .= '</frame>';
	$xml .= '</manialink>';
	$xml .= '</manialinks>';

	$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($player->login, $xml, $iwid_config['WIDGET'][0]['AUTO_HIDE'][0], false));
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function iwid_hideMessageWindow ($aseco, $player = false) {
	global $iwid_config, $IWID_MSGS;

	// If the Window is already hidden, bail out.
	if ($player->data['InfoWidgetStatus'] == 'hidden') {
		return;
	}

	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks id="'. $iwid_config['MANIALINK_ID'] .'00">';
	$xml .= '<manialink id="'. $iwid_config['MANIALINK_ID'] .'01">';
	$xml .= '</manialink>';
	$xml .= '</manialinks>';


	if ($player == false) {
		$aseco->client->addCall('SendDisplayManialinkPage', array($xml, 0, false));
	}
	else {
		$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($player->login, $xml, 0, false));
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function chat_toggleinfo ($aseco, $command) {

	if ($command['author']->data['InfoWidgetStatus'] == 'hidden') {
		$command['author']->data['InfoWidgetStatus'] = 'visible';
		iwid_displayMessageWindow($aseco, $command['author']);

		$message = '{#server}> $0F3Your message window is now shown, type {#highlite}/toggleinfo$0F3 to hide them.';
		$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $command['author']->login);
	}
	else {
		iwid_hideMessageWindow($aseco, $command['author']);
		$command['author']->data['InfoWidgetStatus'] = 'hidden';

		$message = '{#server}> $0F3Your message window is now hidden, type {#highlite}/toggleinfo$0F3 to reactivate them.';
		$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $command['author']->login);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function iwid_onPlayerManialinkPageAnswer ($aseco, $answer) {
	global $sn_config;

	if ($answer[2] == 382009003) {

		// Get Player
		$player = $aseco->server->players->getPlayer($answer[1]);

		// Player has pressed key F7 (same ManialinkId as plugin.fufi.widgets.php)
		if ($player->data['InfoWidgetStatus'] == 'visible') {
			// Hide the widget
			iwid_hideMessageWindow($aseco, $player);
			$player->data['InfoWidgetStatus'] = 'hidden';
		}
		else {
			// Show the widget
			$player->data['InfoWidgetStatus'] = 'visible';
			iwid_displayMessageWindow($aseco, $player);
		}

	}

}

?>
